"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.compileAndTransform = void 0;
const node_elm_compiler_1 = require("node-elm-compiler");
const fs = __importStar(require("fs"));
const path = __importStar(require("path"));
const parseElm_1 = require("./parseElm");
const typescript_1 = __importDefault(require("typescript"));
const variantShapes_1 = require("./transforms/variantShapes");
const types_1 = require("./types");
const inlineWrappedFunctions_1 = require("./transforms/inlineWrappedFunctions");
const inlineListFromArray_1 = require("./transforms/inlineListFromArray");
// import { prepackFileSync } from 'prepack';
const Terser = __importStar(require("terser"));
const child_process_1 = require("child_process");
const inlineEquality_1 = require("./transforms/inlineEquality");
const modernizeJS_1 = require("./transforms/modernizeJS");
const removeUnusedLocals_1 = require("./transforms/removeUnusedLocals");
const passUnwrappedFunctions_1 = require("./transforms/passUnwrappedFunctions");
const correctVirtualDom_1 = require("./transforms/correctVirtualDom");
const inlineNumberToString_1 = require("./transforms/inlineNumberToString");
exports.compileAndTransform = async (dir, file, options, transforms) => {
    // Compile examples in `testcases/*` folder as js
    // Run whatever transformations we want on them, saving steps as `elm.{transformation}.js`
    if (options.compile) {
        node_elm_compiler_1.compileSync([file], {
            output: 'output/elm.js',
            cwd: dir,
        });
        node_elm_compiler_1.compileSync([file], {
            output: 'output/elm.opt.js',
            cwd: dir,
            optimize: true,
        });
    }
    const pathInOutput = (p) => path.join(dir, 'output', p);
    const elmSource = fs.readFileSync(path.join(dir, file), 'utf8');
    let parsedVariants = parseElm_1.parseElm({
        author: 'author',
        project: 'project',
        source: elmSource,
    }).concat(parseElm_1.primitives);
    let parsed = parseElm_1.parseDir('elm-packages');
    parsedVariants = parsedVariants.concat(parsed);
    let source = typescript_1.default.createSourceFile('elm.js', fs.readFileSync(pathInOutput('elm.opt.js'), 'utf-8'), typescript_1.default.ScriptTarget.ES2018);
    const normalizeVariantShapes = variantShapes_1.createCustomTypesTransformer(parsedVariants, types_1.Mode.Prod);
    // We have to ensure that this transformation takes place before everything else
    if (transforms.replaceVDomNode) {
        const results = typescript_1.default.transform(source, [correctVirtualDom_1.replaceVDomNode()]);
        source = results.transformed[0];
    }
    let inlineCtx;
    const transformations = removeDisabled([
        [transforms.variantShapes, normalizeVariantShapes],
        [
            transforms.inlineFunctions,
            inlineWrappedFunctions_1.createFunctionInlineTransformer(ctx => {
                inlineCtx = ctx;
                reportInlineTransformResult(ctx);
            }),
        ],
        [transforms.inlineEquality, inlineEquality_1.inlineEquality()],
        [transforms.inlineNumberToString, inlineNumberToString_1.inlineNumberToString()],
        [
            transforms.listLiterals == types_1.InlineLists.AsObjects,
            inlineListFromArray_1.createInlineListFromArrayTransformer(inlineListFromArray_1.InlineMode.UsingLiteralObjects(types_1.Mode.Prod)),
        ],
        [
            transforms.listLiterals == types_1.InlineLists.AsCons,
            inlineListFromArray_1.createInlineListFromArrayTransformer(inlineListFromArray_1.InlineMode.UsingConsFunc),
        ],
        [
            transforms.passUnwrappedFunctions,
            passUnwrappedFunctions_1.createPassUnwrappedFunctionsTransformer(() => inlineCtx),
        ],
        [
            !!transforms.objectUpdate,
            transforms.objectUpdate && modernizeJS_1.objectUpdate(transforms.objectUpdate),
        ],
        [transforms.arrowFns, modernizeJS_1.convertFunctionExpressionsToArrowFuncs],
        [transforms.unusedValues, removeUnusedLocals_1.createRemoveUnusedLocalsTransform()],
    ]);
    const { transformed: [result], } = typescript_1.default.transform(source, transformations);
    const printer = typescript_1.default.createPrinter();
    fs.writeFileSync(pathInOutput('elm.opt.transformed.js'), printer.printFile(result));
    const initialJs = typescript_1.default.createSourceFile('elm.js', fs.readFileSync(pathInOutput('elm.opt.js'), 'utf-8'), typescript_1.default.ScriptTarget.ES2018);
    fs.writeFileSync(pathInOutput('elm.opt.js'), printer.printFile(initialJs));
    // Prepack, minify, and gzip
    // if (false) {
    //   // if (transforms.prepack) {
    //   const { code } = prepackFileSync([pathInOutput('elm.opt.transformed.js')], {
    //     debugNames: true,
    //     inlineExpressions: true,
    //     maxStackDepth: 1200, // that didn't help
    //   });
    //   fs.writeFileSync(pathInOutput('elm.opt.prepack.js'), code);
    //   if (options.minify) {
    //     await minify(
    //       pathInOutput('elm.opt.prepack.js'),
    //       pathInOutput('elm.opt.prepack.min.js')
    //     );
    //   }
    //   if (options.gzip) {
    //     gzip(pathInOutput('elm.opt.prepack.min.js'));
    //   }
    // }
    if (options.minify) {
        await minify(pathInOutput('elm.opt.js'), pathInOutput('elm.opt.min.js'));
        await minify(pathInOutput('elm.opt.transformed.js'), pathInOutput('elm.opt.transformed.min.js'));
    }
    if (options.gzip) {
        gzip(pathInOutput('elm.opt.min.js'));
        gzip(pathInOutput('elm.opt.transformed.min.js'));
    }
    return {};
};
function removeDisabled(list) {
    let newList = [];
    list.forEach(([cond, val]) => {
        if (![null, false, undefined].includes(cond)) {
            newList.push(val);
        }
    });
    return newList;
}
async function minify(inputFilename, outputFilename) {
    const compress = {
        toplevel: true,
        mangle: false,
        compress: {
            pure_getters: true,
            keep_fargs: false,
            unsafe_comps: true,
            unsafe: true,
            pure_funcs: [
                'F2',
                'F3',
                'F4',
                'F5',
                'F6',
                'F7',
                'F8',
                'F9',
                'A2',
                'A3',
                'A4',
                'A5',
                'A6',
                'A7',
                'A8',
                'A9',
            ],
        },
    };
    const mangle = {
        mangle: true,
        compress: false,
    };
    const input = fs.readFileSync(inputFilename, 'utf8');
    const compressed = await Terser.minify(input, compress);
    let mangled = null;
    if (compressed && compressed.code) {
        mangled = await Terser.minify(compressed.code, mangle);
    }
    else {
        console.log('Error compressing with Terser');
    }
    // console.log('mangled', mangled.error);
    if (mangled && mangled.code) {
        fs.writeFileSync(outputFilename, mangled.code);
    }
    else {
        console.log('Error mangling with Terser');
    }
}
async function gzip(file) {
    // --keep = keep the original file
    // --force = overwrite the exisign gzip file if it's there
    child_process_1.execSync('gzip --keep --force ' + file);
}
function reportInlineTransformResult(ctx) {
    const { splits, partialApplications, inlined } = ctx;
    console.log(`functionInlineTransformer: splitCount=${splits.size}, partialApplicationCount=${partialApplications.size}, inlined=`, inlined
    // splits
    );
}
//# sourceMappingURL=compile.js.map