"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
// tslint:disable-next-line no-require-imports no-var-requires
const commander_1 = __importDefault(require("commander"));
const path = __importStar(require("path"));
const Transform = __importStar(require("./transform"));
const types_1 = require("./types");
const node_elm_compiler_1 = require("node-elm-compiler");
const fs = __importStar(require("fs"));
const chalk_1 = __importDefault(require("chalk"));
const { version } = require('../package.json');
const BenchInit = __importStar(require("./benchmark/init"));
const Benchmark = __importStar(require("./benchmark/benchmark"));
const Reporting = __importStar(require("./benchmark/reporting"));
commander_1.default
    .version(version)
    .description(`${chalk_1.default.yellow('Elm Optimize Level 2!')}
    
This applies a second level of optimization to the javascript that Elm creates.

Make sure you're familiar with Elm's built-in optimization first: ${chalk_1.default.cyan('https://guide.elm-lang.org/optimization/asset_size.html')}

Give me an Elm file, I'll compile it behind the scenes using Elm 0.19.1, and then I'll make some more optimizations!`)
    .usage('[options] <src/Main.elm>')
    .option('--output <output>', 'the javascript file to create.', 'elm.js')
    .option('-O3, --optimize-speed', 'Enable optimizations that likely increases asset size', false)
    .option('--init-benchmark <dir>', 'Generate some files to help run benchmarks')
    .option('--benchmark <dir>', 'Run the benchmark in the given directory.')
    .option('--replacements <dir>', 'Replace stuff')
    .parse(process.argv);
async function run(inputFilePath) {
    const dirname = process.cwd();
    let jsSource = '';
    let elmFilePath = undefined;
    const options = commander_1.default.opts();
    const replacements = options.replacements;
    const o3Enabled = options.optimizeSpeed;
    if (commander_1.default.initBenchmark) {
        console.log(`Initializing benchmark ${commander_1.default.initBenchmark}`);
        BenchInit.generate(commander_1.default.initBenchmark);
        process.exit(0);
    }
    if (commander_1.default.benchmark) {
        const options = {
            compile: true,
            gzip: true,
            minify: true,
            verbose: true,
            assetSizes: true,
            runBenchmark: [
                {
                    browser: types_1.Browser.Chrome,
                    headless: true,
                }
            ],
            transforms: types_1.benchmarkDefaults(o3Enabled, replacements),
        };
        const report = await Benchmark.run(options, [
            {
                name: 'Benchmark',
                dir: commander_1.default.benchmark,
                elmFile: 'V8/Benchmark.elm',
            }
        ]);
        console.log(Reporting.terminal(report));
        //       fs.writeFileSync('./results.markdown', Reporting.markdownTable(result));
        process.exit(0);
    }
    if (inputFilePath && inputFilePath.endsWith('.js')) {
        jsSource = fs.readFileSync(inputFilePath, 'utf8');
        console.log('Optimizing existing JS...');
    }
    else if (inputFilePath && inputFilePath.endsWith('.elm')) {
        elmFilePath = inputFilePath;
        jsSource = node_elm_compiler_1.compileToStringSync([inputFilePath], {
            output: 'output/elm.opt.js',
            cwd: dirname,
            optimize: true,
            processOpts: 
            // ignore stdout
            {
                stdio: ['inherit', 'ignore', 'inherit'],
            },
        });
        if (jsSource != '') {
            console.log('Compiled, optimizing JS...');
        }
        else {
            process.exit(1);
        }
    }
    else {
        console.error('Please provide a path to an Elm file.');
        commander_1.default.outputHelp();
        return;
    }
    if (jsSource != '') {
        const transformed = await Transform.transform(dirname, jsSource, elmFilePath, false, types_1.toolDefaults(o3Enabled, replacements));
        // Make sure all the folders up to the output file exist, if not create them.
        // This mirrors elm make behavior.
        const outputDirectory = path.dirname(commander_1.default.output);
        if (!fs.existsSync(outputDirectory)) {
            fs.mkdirSync(outputDirectory, { recursive: true });
        }
        fs.writeFileSync(commander_1.default.output, transformed);
        const fileName = path.basename(inputFilePath);
        console.log('Success!');
        console.log('');
        console.log(`   ${fileName} ───> ${commander_1.default.output}`);
        console.log('');
    }
}
run(commander_1.default.args[0]).catch((e) => console.error(e));
//# sourceMappingURL=index.js.map